"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("./logger");
const _connectors = {};
exports.PitayaDataFlow = {
    async connect(connector) {
        try {
            const { name } = connector.constructor;
            if (name in _connectors) {
                throw new Error(`Connector of type {${name}} already defined.`);
            }
            await connector.initialize();
            connector.name = name;
            _connectors[connector.constructor.name] = connector;
        }
        catch (error) {
            logger_1.default.error("PitayaDataFlow.connect()", error);
        }
    },
    async disconnect({ name }) {
        try {
            if (!(name in _connectors)) {
                throw new Error(`Connector of type {${name}} not defined.`);
            }
            await _connectors[name].destroy();
            delete _connectors[name];
        }
        catch (error) {
            logger_1.default.error("PitayaDataFlow.disconnect()", error);
        }
    },
    from({ name }) {
        try {
            if (!(name in _connectors)) {
                throw new Error(`Connector of type {${name}} not defined.`);
            }
            return _connectors[name];
        }
        catch (error) {
            logger_1.default.error("PitayaDataFlow.from()", error);
        }
        return undefined;
    },
    has({ name }) {
        return _connectors.hasOwnProperty(name);
    }
};
//# sourceMappingURL=data-flow.js.map