"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events = {
    back: [],
    forward: [],
    default: [],
};
const detectBackOrForward = function (onBack, onForward, onDefault) {
    const hashHistory = [window.location.hash];
    let historyLength = window.history.length;
    return function () {
        const sHash = window.location.hash;
        const length = window.history.length;
        if (hashHistory.length && historyLength === length) {
            if (hashHistory[hashHistory.length - 2] === sHash) {
                hashHistory.pop();
                onBack();
            }
            else {
                hashHistory.push(sHash);
                onForward();
            }
        }
        else {
            hashHistory.push(sHash);
            historyLength = length;
            onDefault();
        }
    };
};
exports.PitayaNavigator = {
    on(eventName, callback) {
        if (!(eventName in events)) {
            throw new Error(`Navigation: [ ${eventName} ] is not a valid event - choose one of the following: 'back' 'forward' 'default'.`);
        }
        events[eventName].push(callback);
    }
};
const navigateBackCallback = function (event) {
    events.back.forEach(callback => callback(event));
};
const navigateForwardCallback = function (event) {
    events.forward.forEach(callback => callback(event));
};
const navigateDefaultCallback = function (event) {
    events.default.forEach(callback => callback(event));
};
window.addEventListener("hashchange", detectBackOrForward(navigateBackCallback, navigateForwardCallback, navigateDefaultCallback));
//# sourceMappingURL=navigator.js.map